package org.globalplatform;

import javacard.framework.Shareable;

/**
 * This interface defines a method to receive a notification upon completion
 * (success or failure) of an Administration Session.<p>
 *
 * An Application that wishes to receive such a notification shall implement the
 * {@link javacard.framework.Applet#getShareableInterfaceObject} to return an
 * {@link AdminSessionListener} instance when the <code>clientAID</code> parameter
 * is set to <code>null</code>, and the <code>parameter</code> parameter is set
 * to {@link #SERVICE_ADMIN_SESSION_REPORT}.
 *
 * @since <ul>
 * <li>export file version 1.8: initial version.</li>
 * </ul>
 */
public interface AdminSessionListener extends Shareable 
{
   /**
    * The parameter to be provided to {@link javacard.framework.Applet#getShareableInterfaceObject} 
	* to retrieve an instance of this interface.
    */
    public static final byte SERVICE_ADMIN_SESSION_REPORT= (byte)0x01;

	/**
	 * Constant notifying that an Administration Session ended successfully
	 */
	public final static short ADMIN_SESSION_NO_ERROR=0x0001;

	/**
	 * Constant notifying that an Administration Session failed. That is, the
	 * retry policy of the session is exhausted and the Administration Session
	 * request is aborted.
	 */
	public final static short ADMIN_SESSION_ERROR=(short) 0x8001; 

	/**
	 * Notifies the Application that the requested Administration Session
	 * completed (successfully or not).  <p>
     *
	 * The OPEN notifies the Application when the Administration Session ends
	 * or when the retry policy is exhausted.<p>
	 * 
	 * @param status Either {@link #ADMIN_SESSION_NO_ERROR} (success) or {@link
	 * #ADMIN_SESSION_ERROR} (failure).
	 */
	public void administrationSessionReport(short status);
}